@@ -6,7 +6,7 @@
  *                                                                          *
  *                              C Header File                               *
  *                                                                          *
- *          Copyright (C) 1992-2003, Free Software Foundation, Inc.         *
+ *          Copyright (C) 1992-2005, Free Software Foundation, Inc.         *
  *                                                                          *
  * GNAT is free software;  you can  redistribute it  and/or modify it under *
  * terms of the  GNU General Public License as published  by the Free Soft- *
@@ -16,8 +16,8 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
  * for  more details.  You should have  received  a copy of the GNU General *
  * Public License  distributed with GNAT;  see file COPYING.  If not, write *
- * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
- * MA 02111-1307, USA.                                                      *
+ * to  the  Free Software Foundation,  51  Franklin  Street,  Fifth  Floor, *
+ * Boston, MA 02110-1301, USA.                                              *
  *                                                                          *
  * As a  special  exception,  if you  link  this file  with other  files to *
  * produce an executable,  this file does not by itself cause the resulting *
@@ -30,13 +30,19 @@
  *                                                                          *
  ****************************************************************************/
 
+
+/* C counterparts of what System.Standard_Library defines.  */
+
+typedef unsigned Exception_Code;
+
 struct Exception_Data
 {
-  char  Handled_By_Others;
+  char Not_Handled_By_Others;
   char Lang;
   int Name_Length;
-  char *Full_Name, Htable_Ptr;
-  int Import_Code;
+  char *Full_Name, *Htable_Ptr;
+  Exception_Code Import_Code;
+  void (*Raise_Hook)(void);
 };
 
 typedef struct Exception_Data *Exception_Id;
@@ -61,9 +67,11 @@ extern void set_gnat_exit_status	(int);
 extern void __gnat_set_globals		(int, int,
 						 char, char, char, char,
 						 char *, char *,
-						 int, int, int, int);
-extern void __gnat_initialize		(void);
+						 int, int, int, int, int, int);
+extern void __gnat_initialize		(void *);
 extern void __gnat_init_float		(void);
 extern void __gnat_install_handler	(void);
+extern void __gnat_install_SEH_handler  (void *);
+extern void __gnat_adjust_context_for_raise (int, void *);
 
 extern int gnat_exit_status;
