@@ -338,28 +338,32 @@ typedef Int Mechanism_Type;
 #define CE_Index_Check_Failed               5
 #define CE_Invalid_Data                     6
 #define CE_Length_Check_Failed              7
-#define CE_Null_Not_Allowed                 8
-#define CE_Overflow_Check_Failed            9
-#define CE_Partition_Check_Failed          10
-#define CE_Range_Check_Failed              11
-#define CE_Tag_Check_Failed                12
-#define PE_Access_Before_Elaboration       13
-#define PE_Accessibility_Check_Failed      14
-#define PE_All_Guards_Closed               15
-#define PE_Duplicated_Entry_Address        16
-#define PE_Explicit_Raise                  17
-#define PE_Finalize_Raised_Exception       18
-#define PE_Misaligned_Address_Value        19
-#define PE_Missing_Return                  20
-#define PE_Overlaid_Controlled_Object      21
-#define PE_Potentially_Blocking_Operation  22
-#define PE_Stubbed_Subprogram_Called       23
-#define PE_Unchecked_Union_Restriction     24
-#define PE_Illegal_RACW_E_4_18             25
-#define SE_Empty_Storage_Pool              26
-#define SE_Explicit_Raise                  27
-#define SE_Infinite_Recursion              28
-#define SE_Object_Too_Large                29
-#define SE_Restriction_Violation           30
+#define CE_Null_Exception_Id                9
+#define CE_Null_Not_Allowed                 9
+#define CE_Overflow_Check_Failed           10
+#define CE_Partition_Check_Failed          11
+#define CE_Range_Check_Failed              12
+#define CE_Tag_Check_Failed                13
 
-#define LAST_REASON_CODE                   30
+#define PE_Access_Before_Elaboration       14
+#define PE_Accessibility_Check_Failed      15
+#define PE_All_Guards_Closed               16
+#define PE_Duplicated_Entry_Address        17
+#define PE_Explicit_Raise                  18
+#define PE_Finalize_Raised_Exception       19
+#define PE_Implicit_Return                 20
+#define PE_Misaligned_Address_Value        21
+#define PE_Missing_Return                  22
+#define PE_Overlaid_Controlled_Object      23
+#define PE_Potentially_Blocking_Operation  24
+#define PE_Stubbed_Subprogram_Called       25
+#define PE_Unchecked_Union_Restriction     26
+#define PE_Illegal_RACW_E_4_18             27
+
+#define SE_Empty_Storage_Pool              28
+#define SE_Explicit_Raise                  29
+#define SE_Infinite_Recursion              30
+#define SE_Object_Too_Large                31
+#define SE_Restriction_Violation           32
+
+#define LAST_REASON_CODE                   31
