@@ -15,8 +15,8 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GCC; see the file COPYING.  If not, write to
-the Free Software Foundation, 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.  */
 
 /* Written by Per Bothner <bothner@cygnus.com>, July 1998. */
 
@@ -37,7 +37,7 @@ buffer_grow (struct buffer *bufp, int size)
     {
       if (size < 120)
 	size = 120;
-      bufp->data = xmalloc (size);
+      bufp->data = XNEWVEC (unsigned char, size);
       bufp->ptr = bufp->data;
     }
   else
