@@ -69,10 +69,10 @@ testit##NAME (void)						\
 }
 
 #ifndef SKIP_COMPLEX_INT
-T(cc, _Complex char, (0,1))
-T(cs, _Complex short, (1,2))
+T(cc, _Complex char, CINT (0, 1))
+T(cs, _Complex short, CINT (1, 2))
 #endif
-T(cf, _Complex float, (6.0,7.0))
+T(cf, _Complex float, CDBL (6.0, 7.0))
 
 #undef T
 
