@@ -3,7 +3,7 @@
    assume something about memory addressing modes.  */
 
 /* { dg-do compile { target i?86-*-* x86_64-*-* } } */
-/* { dg-options "-O1 -fdump-tree-vars" } */
+/* { dg-options "-O1 -fdump-tree-optimized" } */
 
 int arr_base[100];
 
@@ -11,7 +11,7 @@ int foo(int);
 
 void xxx(void)
 {
-  int iter;
+  long iter;
 
   for (iter = 0; iter < 100; iter++)
     arr_base[iter] = foo (iter);
@@ -20,8 +20,11 @@ void xxx(void)
 /* Access to arr_base[iter].y should not be strength reduced, since
    we have a memory mode including multiplication by 4.  */
 
-/* { dg-final { scan-tree-dump-times "arr_base.*=" 1 "vars" } } */
+/* { dg-final { scan-tree-dump-times "MEM" 1 "optimized" } } */
+/* { dg-final { scan-tree-dump-times "step:" 1 "optimized" } } */
 
 /* And original induction variable should be preserved.  */
 
-/* { dg-final { scan-tree-dump-times "int iter" 1 "vars" } } */
+/* { dg-final { scan-tree-dump-times "int iter" 1 "optimized" } } */
+
+/* { dg-final { cleanup-tree-dump "optimized" } } */
