@@ -2,7 +2,7 @@
 ! Verify that the D format uses 'D' as the exponent character.
 !   "     "    "  E   "     "   'E' "   "     "         " 
 CHARACTER*10 c1, c2
-REAL*8 r
+REAL(kind=8) r
 r = 1.0
 write(c1,"(e9.2)") r
 write(c2,"(d9.2)") r
