@@ -1,3 +1,6 @@
+// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
+// reminder from Petur Runolfsson <peturr02@ru.is>.
+
 // Copyright (C) 2003 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -17,8 +20,6 @@
 // USA.
 
 #include <iostream>
-#include <cstdio>
-#include <cwchar>
 #include <testsuite_hooks.h>
 
 void
@@ -26,12 +27,13 @@ test01()
 {
   std::freopen("cin_unget-1.txt", "r", stdin);
 
-  wchar_t buf[2];
-  VERIFY( std::wcin.rdbuf()->sgetn(buf, 2) == 2 );
-  wint_t c1 = std::wcin.rdbuf()->sungetc();
-  wint_t c2 = std::fgetwc(stdin);
-  VERIFY( c1 == std::char_traits<wchar_t>::to_int_type(buf[1]) );
-  VERIFY( c2 == c1 );
+  wchar_t c1;
+  wchar_t c2;
+  std::wcin.get(c1);
+  std::wcin.unget();
+  std::wcin.get(c2);
+  VERIFY( std::wcin.good() );
+  VERIFY( c1 == c2 );
 }
 
 int main(void)
