@@ -16,47 +16,39 @@
 // Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
-// The ARM simulator does not provide support for "fstat", which
-// causes "sbumpc" to return an incorrect value.
-// { dg-do run { xfail arm*-*-elf arm*-*-eabi } }
-
 #include <cstdio>
 #include <fstream>
 #include <ext/stdio_filebuf.h>
 #include <testsuite_hooks.h>
 
-void test2()
+void test1()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
+  FILE* file = fopen("tmp_10063-1", "w");
+  putc('0', file);
+  putc('1', file);
+  {
+    __gnu_cxx::stdio_filebuf<char> sbuf(file, ios_base::out);
+    sbuf.sputc('2');
+    sbuf.sputc('3');
+  }
+  putc('4', file);	
+  fclose(file);
+
   filebuf fbuf;
-  fbuf.open("tmp_10063-2", ios_base::out | ios_base::trunc);
-  fbuf.sputn("01234", 5);
+  fbuf.open("tmp_10063-1", ios_base::in);	
+  char buf[10];
+  streamsize n = fbuf.sgetn(buf, sizeof(buf));		
   fbuf.close();
 
-  FILE* file = fopen("tmp_10063-2", "r");
-  setbuf(file, NULL);
-  int c = getc(file);
-  VERIFY(c == '0');
-  c = getc(file);
-  VERIFY(c == '1');
-  {
-    __gnu_cxx::stdio_filebuf<char> sbuf(file, ios_base::in);
-    c = sbuf.sbumpc();
-    VERIFY(c == '2');
-    c = sbuf.sbumpc();
-    VERIFY(c == '3');
-    c = sbuf.sbumpc();
-    VERIFY(c == '4');
-    c = sbuf.sgetc();
-    VERIFY(c == EOF);
-  }
-  fclose(file);
+  VERIFY(n == 5);
+  VERIFY(!memcmp(buf, "01234", 5)); 
 }
 
 int main()
 {
-  test2();
+  test1();
   return 0;
 }
