@@ -1,6 +1,6 @@
 // 2004-10-20  Benjamin Kosnik  <bkoz@redhat.com>
 //
-// Copyright (C) 2004 Free Software Foundation, Inc.
+// Copyright (C) 2004, 2005, 2006 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -15,13 +15,14 @@
 //
 // You should have received a copy of the GNU General Public License along
 // with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 // USA.
 
 // 6.2.2 Class template array
 
 #include <tr1/array>
 #include <stdexcept>
+#include <testsuite_hooks.h>
 
 void
 test01() 
@@ -34,15 +35,17 @@ test01()
   try
     {
       a.at(len);
+      VERIFY( false );
     }
   catch(std::out_of_range& obj)
     {
       // Expected.
+      VERIFY( true );
     }
   catch(...)
     {
       // Failed.
-      throw;
+      VERIFY( false );
     }
 }
 
@@ -51,4 +54,3 @@ int main()
   test01();
   return 0;
 }
-
